package at.herold.test.objlifetime.viewer;

import java.awt.Color;
import java.awt.Graphics;
/**
 * Analyse the number of life objects in the statistics.
 * @author Peter Kofler
 */
public class LifeObjectsPerTimeUnitAnalyser extends LifeTimeAnalyser
{
   private static final int MAX_X_PIXEL = -1;
   private static final int MAX_Y_PIXEL = 700;
   
   private int startTimeUnit;
   private int durationUnits;

   /** Array to count number of life elements. */
   private int[] lifeObjectsPerTimeUnit;

   private int minLifeObjects;
   private int maxLifeObjects;
   double timeUnitsPerXPixel;
   double lifeObjectxPerYPixel;
   
   public void analyse(ViewedLifetimeStatistics pData)
   {
      data = pData;
      startTimeUnit = data.getMinTimeStamp();
      durationUnits = data.getMaxTimeStamp() - startTimeUnit + 1;
      lifeObjectsPerTimeUnit = new int[durationUnits];

      sumLifeObjectsForTimeUnits();

      calcMinMaxOfLifeObjects();
      
      if (MAX_X_PIXEL==-1) timeUnitsPerXPixel = 1.0;
      else timeUnitsPerXPixel = durationUnits/1.0/MAX_X_PIXEL;
      lifeObjectxPerYPixel = maxLifeObjects/1.0/MAX_Y_PIXEL;
   }

   private void sumLifeObjectsForTimeUnits()
   {
      final ViewedLifetime[] lives = data.getLifetimeArray();
      for (int i = 0; i < lives.length; i++)
      {
         final ViewedLifetime lt = lives[i];
      
         // increment for each time unit for each object alive
         for (int j = lt.startTimeStamp; j <= lt.endTimeStamp; j++)
         {
            lifeObjectsPerTimeUnit[j - startTimeUnit]++;
         }
      
      }
   }

   private void calcMinMaxOfLifeObjects()
   {
      minLifeObjects = Integer.MAX_VALUE;
      maxLifeObjects = 0;
      for (int i=0;i<lifeObjectsPerTimeUnit.length;i++) {
         final int val = lifeObjectsPerTimeUnit[i];
         if (val<minLifeObjects) minLifeObjects = val;
         if (val>maxLifeObjects) maxLifeObjects = val;
      }
   }

   public void printResults() {
      System.out.println("-------------------- " +this.getClass().getName()+" --------------------");

      System.out.println(durationUnits+" time units run.");
      System.out.println("time from "+data.getMinTimeStamp()+" to "+data.getMaxTimeStamp());

      System.out.println(minLifeObjects+" min number of life objects per time unit.");
      System.out.println(maxLifeObjects+" max number of life objects per time unit.");
      System.out.println(data.getNumberEvents()+" total number of life objects.");
   }

   public int getPaintYAxis()
   {
      return (int) (maxLifeObjects/lifeObjectxPerYPixel);
   }

   public int getPaintXAxis()
   {
      return (int) (durationUnits/timeUnitsPerXPixel);
   }

   public void paintResults(Graphics g) {
      g.setColor(Color.BLACK);
      for (int i=0;i<lifeObjectsPerTimeUnit.length;i++) {
         final int x = (int) (i/timeUnitsPerXPixel);
         final int val = (int) ( lifeObjectsPerTimeUnit[i]/lifeObjectxPerYPixel);
         final int y = MAX_Y_PIXEL-val;
         g.drawLine(x, MAX_Y_PIXEL, x, y);
      }
      
      g.setColor(Color.RED);
      g.drawString("^ y-axis = # life objects ("+minLifeObjects+" min./"+maxLifeObjects+" max.)",10,16);
      g.drawString("> x-axis = time units ("+durationUnits+" units/"+durationUnits/100+" sec.)",10,MAX_Y_PIXEL-4);
   }
   
}
